@@ -6,14 +6,21 @@ import sys
 from nose.tools import assert_equal, assert_false, assert_raises, assert_true
 
 from voluptuous import (
-    Schema, Required, Exclusive, Optional, Extra, Invalid, In, Remove, Literal,
-    Url, MultipleInvalid, LiteralInvalid, TypeInvalid, NotIn, Match, Email,
+    Schema, Required, Exclusive, Inclusive, Optional, Extra, Invalid, In, Remove,
+    Literal, Url, MultipleInvalid, LiteralInvalid, TypeInvalid, NotIn, Match, Email,
     Replace, Range, Coerce, All, Any, Length, FqdnUrl, ALLOW_EXTRA, PREVENT_EXTRA,
     validate, ExactSequence, Equal, Unordered, Number, Maybe, Datetime, Date,
     Contains, Marker, IsDir, IsFile, PathExists, SomeOf, TooManyValid, Self,
-    raises)
+    raises, Union, Clamp)
 from voluptuous.humanize import humanize_error
-from voluptuous.util import u
+from voluptuous.util import u, Capitalize, Lower, Strip, Title, Upper
+
+
+def test_new_required_test():
+    schema = Schema({
+        'my_key': All(int, Range(1, 20)),
+    }, required=True)
+    assert_true(schema.required)
 
 
 def test_exact_sequence():
@@ -372,6 +379,7 @@ def test_schema_extend():
     assert extended.schema == {'a': int, 'b': str}
     assert extended.required == base.required
     assert extended.extra == base.extra
+    assert isinstance(extended, Schema)
 
 
 def test_schema_extend_overrides():
@@ -411,6 +419,20 @@ def test_subschema_extension():
     assert_equal(extended.schema, {'a': {'b': str, 'c': float, 'e': int}, 'd': str})
 
 
+def test_schema_extend_handles_schema_subclass():
+    """Verify that Schema.extend handles a subclass of Schema"""
+    class S(Schema):
+        pass
+
+    base = S({Required('a'): int})
+    extension = {Optional('b'): str}
+    extended = base.extend(extension)
+
+    expected_schema = {Required('a'): int, Optional('b'): str}
+    assert extended.schema == expected_schema
+    assert isinstance(extended, S)
+
+
 def test_equality():
     assert_equal(Schema('foo'), Schema('foo'))
 
@@ -563,11 +585,97 @@ def test_fix_157():
     assert_raises(MultipleInvalid, s, ['four'])
 
 
-def test_range_exlcudes_nan():
+def test_range_inside():
+    s = Schema(Range(min=0, max=10))
+    assert_equal(5, s(5))
+
+
+def test_range_outside():
+    s = Schema(Range(min=0, max=10))
+    assert_raises(MultipleInvalid, s, 12)
+    assert_raises(MultipleInvalid, s, -1)
+
+
+def test_range_no_upper_limit():
+    s = Schema(Range(min=0))
+    assert_equal(123, s(123))
+
+
+def test_range_excludes_nan():
     s = Schema(Range(min=0, max=10))
     assert_raises(MultipleInvalid, s, float('nan'))
 
 
+def test_range_excludes_none():
+    s = Schema(Range(min=0, max=10))
+    assert_raises(MultipleInvalid, s, None)
+
+
+def test_range_excludes_string():
+    s = Schema(Range(min=0, max=10))
+    assert_raises(MultipleInvalid, s, "abc")
+
+
+def test_range_excludes_unordered_object():
+    class MyObject(object):
+        pass
+
+    s = Schema(Range(min=0, max=10))
+    assert_raises(MultipleInvalid, s, MyObject())
+
+
+def test_clamp_inside():
+    s = Schema(Clamp(min=1, max=10))
+    assert_equal(5, s(5))
+
+
+def test_clamp_above():
+    s = Schema(Clamp(min=1, max=10))
+    assert_equal(10, s(12))   
+
+
+def test_clamp_below():
+    s = Schema(Clamp(min=1, max=10))
+    assert_equal(1, s(-3)) 
+
+
+def test_clamp_invalid():
+    s = Schema(Clamp(min=1, max=10))
+    if sys.version_info.major >= 3:
+        assert_raises(MultipleInvalid, s, None)
+        assert_raises(MultipleInvalid, s, "abc")
+    else:
+        assert_equal(1, s(None))
+
+
+def test_length_ok():
+    v1 = ['a', 'b', 'c']
+    s = Schema(Length(min=1, max=10))
+    assert_equal(v1, s(v1))
+    v2 = "abcde"
+    assert_equal(v2, s(v2))
+
+
+def test_length_too_short():
+    v1 = []
+    s = Schema(Length(min=1, max=10))
+    assert_raises(MultipleInvalid, s, v1)
+    v2 = ''
+    assert_raises(MultipleInvalid, s, v2)
+
+
+def test_length_too_long():
+    v = ['a', 'b', 'c']
+    s = Schema(Length(min=0, max=2))
+    assert_raises(MultipleInvalid, s, v) 
+
+
+def test_length_invalid():
+    v = None
+    s = Schema(Length(min=0, max=2))
+    assert_raises(MultipleInvalid, s, v) 
+
+
 def test_equal():
     s = Schema(Equal(1))
     s(1)
@@ -624,6 +732,12 @@ def test_maybe():
     assert_raises(Invalid, s, {'foo': 'bar'})
 
 
+def test_maybe_accepts_msg():
+    s = Schema(Maybe(int, msg='int or None expected'))
+    with raises(MultipleInvalid, 'int or None expected'):
+        assert s([])
+
+
 def test_empty_list_as_exact():
     s = Schema([])
     assert_raises(Invalid, s, [1])
@@ -861,10 +975,15 @@ def test_datetime():
 def test_date():
     schema = Schema({"date": Date()})
     schema({"date": "2016-10-24"})
-    assert_raises(MultipleInvalid, schema, {"date": "2016-10-2"})
     assert_raises(MultipleInvalid, schema, {"date": "2016-10-24Z"})
 
 
+def test_date_custom_format():
+    schema = Schema({"date": Date("%Y%m%d")})
+    schema({"date": "20161024"})
+    assert_raises(MultipleInvalid, schema, {"date": "2016-10-24"})
+
+
 def test_ordered_dict():
     if not hasattr(collections, 'OrderedDict'):
         # collections.OrderedDict was added in Python2.7; only run if present
@@ -1030,6 +1149,9 @@ def test_description():
     exclusive = Exclusive('alpha', 'angles', description='Hello')
     assert exclusive.description == 'Hello'
 
+    inclusive = Inclusive('alpha', 'angles', description='Hello')
+    assert inclusive.description == 'Hello'
+
     required = Required('key', description='Hello')
     assert required.description == 'Hello'
 
@@ -1085,6 +1207,53 @@ def test_self_validation():
     schema({"follow": {"follow": {"number": 123456}}})
 
 
+def test_any_error_has_path():
+    """https://github.com/alecthomas/voluptuous/issues/347"""
+    s = Schema({
+        Optional('q'): int,
+        Required('q2'): Any(int, msg='toto')
+    })
+    try:
+        s({'q': 'str', 'q2': 'tata'})
+    except MultipleInvalid as exc:
+        assert (
+            (exc.errors[0].path == ['q'] and exc.errors[1].path == ['q2']) or
+            (exc.errors[1].path == ['q'] and exc.errors[0].path == ['q2'])
+        )
+    else:
+        assert False, "Did not raise AnyInvalid"
+
+
+def test_all_error_has_path():
+    """https://github.com/alecthomas/voluptuous/issues/347"""
+    s = Schema({
+        Optional('q'): int,
+        Required('q2'): All([str, Length(min=10)], msg='toto'),
+    })
+    try:
+        s({'q': 'str', 'q2': 12})
+    except MultipleInvalid as exc:
+        assert (
+            (exc.errors[0].path == ['q'] and exc.errors[1].path == ['q2']) or
+            (exc.errors[1].path == ['q'] and exc.errors[0].path == ['q2'])
+        )
+    else:
+        assert False, "Did not raise AllInvalid"
+
+
+def test_match_error_has_path():
+    """https://github.com/alecthomas/voluptuous/issues/347"""
+    s = Schema({
+        Required('q2'): Match("a"),
+    })
+    try:
+        s({'q2': 12})
+    except MultipleInvalid as exc:
+        assert exc.errors[0].path == ['q2']
+    else:
+        assert False, "Did not raise MatchInvalid"
+
+
 def test_self_any():
     schema = Schema({"number": int,
                      "follow": Any(Self, "stop")})
@@ -1141,3 +1310,216 @@ def test_SomeOf_on_bounds_assertion():
 
 def test_comparing_voluptuous_object_to_str():
     assert_true(Optional('Classification') < 'Name')
+
+
+def test_set_of_integers():
+    schema = Schema({int})
+    with raises(Invalid, 'expected a set'):
+        schema(42)
+    with raises(Invalid, 'expected a set'):
+        schema(frozenset([42]))
+
+    schema(set())
+    schema(set([42]))
+    schema(set([42, 43, 44]))
+    try:
+        schema(set(['abc']))
+    except MultipleInvalid as e:
+        assert_equal(str(e), "invalid value in set")
+    else:
+        assert False, "Did not raise Invalid"
+
+
+def test_frozenset_of_integers():
+    schema = Schema(frozenset([int]))
+    with raises(Invalid, 'expected a frozenset'):
+        schema(42)
+    with raises(Invalid, 'expected a frozenset'):
+        schema(set([42]))
+
+    schema(frozenset())
+    schema(frozenset([42]))
+    schema(frozenset([42, 43, 44]))
+    try:
+        schema(frozenset(['abc']))
+    except MultipleInvalid as e:
+        assert_equal(str(e), "invalid value in frozenset")
+    else:
+        assert False, "Did not raise Invalid"
+
+
+def test_set_of_integers_and_strings():
+    schema = Schema({int, str})
+    with raises(Invalid, 'expected a set'):
+        schema(42)
+
+    schema(set())
+    schema(set([42]))
+    schema(set(['abc']))
+    schema(set([42, 'abc']))
+    try:
+        schema(set([None]))
+    except MultipleInvalid as e:
+        assert_equal(str(e), "invalid value in set")
+    else:
+        assert False, "Did not raise Invalid"
+
+
+def test_frozenset_of_integers_and_strings():
+    schema = Schema(frozenset([int, str]))
+    with raises(Invalid, 'expected a frozenset'):
+        schema(42)
+
+    schema(frozenset())
+    schema(frozenset([42]))
+    schema(frozenset(['abc']))
+    schema(frozenset([42, 'abc']))
+    try:
+        schema(frozenset([None]))
+    except MultipleInvalid as e:
+        assert_equal(str(e), "invalid value in frozenset")
+    else:
+        assert False, "Did not raise Invalid"
+
+
+def test_lower_util_handles_various_inputs():
+    assert Lower(3) == "3"
+    assert Lower(u"3") == u"3"
+    assert Lower(b'\xe2\x98\x83'.decode("UTF-8")) == b'\xe2\x98\x83'.decode("UTF-8")
+    assert Lower(u"A") == u"a"
+
+
+def test_upper_util_handles_various_inputs():
+    assert Upper(3) == "3"
+    assert Upper(u"3") == u"3"
+    assert Upper(b'\xe2\x98\x83'.decode("UTF-8")) == b'\xe2\x98\x83'.decode("UTF-8")
+    assert Upper(u"a") == u"A"
+
+
+def test_capitalize_util_handles_various_inputs():
+    assert Capitalize(3) == "3"
+    assert Capitalize(u"3") == u"3"
+    assert Capitalize(b'\xe2\x98\x83'.decode("UTF-8")) == b'\xe2\x98\x83'.decode("UTF-8")
+    assert Capitalize(u"aaa aaa") == u"Aaa aaa"
+
+
+def test_title_util_handles_various_inputs():
+    assert Title(3) == "3"
+    assert Title(u"3") == u"3"
+    assert Title(b'\xe2\x98\x83'.decode("UTF-8")) == b'\xe2\x98\x83'.decode("UTF-8")
+    assert Title(u"aaa aaa") == u"Aaa Aaa"
+
+
+def test_strip_util_handles_various_inputs():
+    assert Strip(3) == "3"
+    assert Strip(u"3") == u"3"
+    assert Strip(b'\xe2\x98\x83'.decode("UTF-8")) == b'\xe2\x98\x83'.decode("UTF-8")
+    assert Strip(u" aaa ") == u"aaa"
+
+
+def test_any_required():
+    schema = Schema(Any({'a': int}, {'b': str}, required=True))
+
+    try:
+        schema({})
+    except MultipleInvalid as e:
+        assert_equal(str(e),
+                     "required key not provided @ data['a']")
+    else:
+        assert False, "Did not raise Invalid for MultipleInvalid"
+
+
+def test_any_required_with_subschema():
+    schema = Schema(Any({'a': Any(float, int)},
+                        {'b': int},
+                        {'c': {'aa': int}},
+                    required=True))
+
+    try:
+        schema({})
+    except MultipleInvalid as e:
+        assert_equal(str(e),
+                     "required key not provided @ data['a']")
+    else:
+        assert False, "Did not raise Invalid for MultipleInvalid"
+
+def test_inclusive():
+    schema = Schema({
+        Inclusive('x', 'stuff'): int,
+        Inclusive('y', 'stuff'): int,
+        })
+
+    r = schema({})
+    assert_equal(r, {})
+
+    r = schema({'x':1, 'y':2})
+    assert_equal(r, {'x':1, 'y':2})
+
+    try:
+        r = schema({'x':1})
+    except MultipleInvalid as e:
+        assert_equal(str(e),
+                     "some but not all values in the same group of inclusion 'stuff' @ data[<stuff>]")
+    else:
+        assert False, "Did not raise Invalid for incomplete Inclusive group"
+
+def test_inclusive_defaults():
+    schema = Schema({
+        Inclusive('x', 'stuff', default=3): int,
+        Inclusive('y', 'stuff', default=4): int,
+        })
+
+    r = schema({})
+    assert_equal(r, {'x':3, 'y':4})
+
+    try:
+        r = schema({'x':1})
+    except MultipleInvalid as e:
+        assert_equal(str(e),
+                     "some but not all values in the same group of inclusion 'stuff' @ data[<stuff>]")
+    else:
+        assert False, "Did not raise Invalid for incomplete Inclusive group with defaults"
+
+def test_exclusive():
+    schema = Schema({
+        Exclusive('x', 'stuff'): int,
+        Exclusive('y', 'stuff'): int,
+        })
+
+    r = schema({})
+    assert_equal(r, {})
+
+    r = schema({'x':1})
+    assert_equal(r, {'x':1})
+
+    try:
+        r = schema({'x':1, 'y': 2})
+    except MultipleInvalid as e:
+        assert_equal(str(e),
+                     "two or more values in the same group of exclusion 'stuff' @ data[<stuff>]")
+    else:
+        assert False, "Did not raise Invalid for multiple values in Exclusive group"
+
+def test_any_with_discriminant():
+    schema = Schema({
+        'implementation': Union({
+            'type': 'A',
+            'a-value': str,
+        }, {
+            'type': 'B',
+            'b-value': int,
+        }, {
+            'type': 'C',
+            'c-value': bool,
+        }, discriminant=lambda value, alternatives: filter(lambda v : v['type'] == value['type'], alternatives))
+    })
+    try:
+        schema({
+            'implementation': {
+            'type': 'C',
+            'c-value': None,}  
+        })
+    except MultipleInvalid as e:
+        assert_equal(str(e),'expected bool for dictionary value @ data[\'implementation\'][\'c-value\']')
+    else:
+        assert False, "Did not raise correct Invalid"
